﻿using HeuristicLab.Common;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Robocode
{
    [StorableClass]
    public class OnBulletHit : CodeNode
    {
        public override int MinimumArity { get { return 2; } }
        public override int MaximumArity { get { return 10; } }

        [Storable]
        public override string Prefix { get; set; }

        [Storable]
        public override string Suffix { get; set; }

        [StorableConstructor]
        private OnBulletHit(bool deserializing) : base(deserializing) { }
        private OnBulletHit(OnBulletHit original, Cloner cloner)
            : base(original, cloner)
        {
            this.Prefix = "\r\npublic void onBulletHit(BulletHitEvent e) {\r\n";
            this.Suffix = "\r\nexecute();\r\n}\r\n";
        }

        public OnBulletHit()
            : base("OnBulletHit", "This method is called when one of your bullets hits another robot.")
        {
            this.Prefix = "\r\npublic void onBulletHit(BulletHitEvent e) {\r\n";
            this.Suffix = "\r\nexecute();\r\n}\r\n";
        }

        public override IDeepCloneable Clone(Cloner cloner)
        {
            return new OnBulletHit(this, cloner);
        }

        public override string Interpret(ISymbolicExpressionTreeNode node, System.Collections.Generic.IEnumerable<ISymbolicExpressionTreeNode> children)
        {
            string result = "";
            foreach (ISymbolicExpressionTreeNode c in children)
                result += "\r\n" + ((CodeNode)c.Symbol).Interpret(c, c.Subtrees);
            return this.Prefix + "\r\n" + result + "\r\n" + this.Suffix;
        }
    }
}