﻿using HeuristicLab.Common;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Robocode
{
    [StorableClass]
    public class OnScannedRobot : CodeNode
    {
        public override int MinimumArity { get { return 2; } }
        public override int MaximumArity { get { return 10; } }

        [Storable]
        public override string Prefix { get; set; }

        [Storable]
        public override string Suffix { get; set; }

        [StorableConstructor]
        private OnScannedRobot(bool deserializing) : base(deserializing) { }
        private OnScannedRobot(OnScannedRobot original, Cloner cloner)
            : base(original, cloner)
        {
            this.Prefix = "\r\npublic void onScannedRobot(ScannedRobotEvent e) {" +
                          "\r\ndouble absoluteBearing = getHeading() + e.getBearing();" +
                          "\r\ndouble bearingFromGun = normalRelativeAngleDegrees(absoluteBearing - getGunHeading());" +
                          "\r\nsetTurnGunRight(bearingFromGun);\r\n";
            this.Suffix = "\r\nexecute();\r\n}\r\n";
        }

        public OnScannedRobot()
            : base("OnScannedRobot", "This method is called when your robot sees another robot, i.e. when the robot's radar scan \"hits\" another robot.")
        {
            this.Prefix = "\r\npublic void onScannedRobot(ScannedRobotEvent e) {" +
                          "\r\ndouble absoluteBearing = getHeading() + e.getBearing();" +
                          "\r\ndouble bearingFromGun = normalRelativeAngleDegrees(absoluteBearing - getGunHeading());" +
                          "\r\nsetTurnGunRight(bearingFromGun);\r\n";
            this.Suffix = "\r\nexecute();\r\n}\r\n";
        }

        public override IDeepCloneable Clone(Cloner cloner)
        {
            return new OnScannedRobot(this, cloner);
        }

        public override string Interpret(ISymbolicExpressionTreeNode node, System.Collections.Generic.IEnumerable<ISymbolicExpressionTreeNode> children)
        {
            string result = "";
            foreach (ISymbolicExpressionTreeNode c in children)
                result += "\r\n" + ((CodeNode)c.Symbol).Interpret(c, c.Subtrees);
            return this.Prefix + "\r\n" + result + "\r\n" + this.Suffix;
        }
    }
}